package main

import (
	"bufio"
	"io"
	"log"
	"strconv"
	"strings"
	"time"

	"github.com/jacobsa/go-serial/serial"
)

// send a command and return the response
func sendCommand(port io.ReadWriteCloser, command string) string {
	// Send command
	log.Printf("> %s\n", command)
	_, err := port.Write(append([]byte(command), '\n'))
	if err != nil {
		log.Fatalf("cannot send command: %v", err)
	}

	// Receive the response
	reader := bufio.NewReader(port)
	line, _, err := reader.ReadLine()
	if err != nil {
		log.Fatalf("cannot receive response: %v", err)
	}
	response := string(line)
	log.Printf("< %s\n", response)

	return response
}

func main() {
	// Set up options.
	options := serial.OpenOptions{
		PortName:              "/dev/ttyUSB0", // for example /dev/ttyUSB0 or COM3
		BaudRate:              115200,
		DataBits:              8,
		StopBits:              1,
		InterCharacterTimeout: 100,
	}

	// Open the port.
	port, err := serial.Open(options)
	if err != nil {
		log.Fatalf("serial.Open: %v", err)
	}

	// Make sure to close the serial port later
	defer port.Close()

	// PD2 = output
	sendCommand(port, "dPD2,o")

	// Use analog reference VCC (5V)
	sendCommand(port, "rVCC")

	// endless loop
	for {
		// LED on
		sendCommand(port, "oPD2,1")
		time.Sleep(500 * time.Millisecond)

		// LED off
		sendCommand(port, "oPD2,0")
		time.Sleep(500 * time.Millisecond)

		// read analog input channel 0
		response := sendCommand(port, "a0")

		// remove "ADC0=" from the repsonse
		response = strings.Replace(response, "ADC0=", "", 1)

		// parse the response as 16 bit hex integer
		intValue, err := strconv.ParseInt(response, 16, 16)
		if err != nil {
			log.Fatalf("cannot parse the response as hex integer: %v", err)
		}

		// calculate the measured voltage
		voltage := float32(intValue) * 5.0 / 1024
		log.Printf("measured %.2f V\n", voltage)
	}

}
